/// <reference path="functions.jsx" />

/**
 * Read a settings file in *.wtf format
 * @param {string} path
 */
function ConfigFile(path) {

    this.file = new File(path);
    this.autoWrite = true;
    this.settings = {};

    // ============================================== //
    this.read();

}

/**
 * Return attribute 'settings' from instance
 */
ConfigFile.prototype.getSettings = function () {
    return this.settings;
}

/**
 * Generate JSON string from settings
 */
ConfigFile.prototype.toJson = function () {
  try {
    return JSON.stringify(this.settings);
  } catch (err) {
    alert("ConfigFile.prototype.toJson: " + err);
  }
}

/**
 * Read wtf formatted config file
 */
ConfigFile.prototype.read = function () {

    try {

        this.settings = {};

        var f = this.file;
        var content = "";

        if (!f.exists) {
            alert("File " + path + " not found");
        }

        f.open('r');

        while (!f.eof) {
            var line = f.readln();

            if (line.length > 0) {

                var parts = line.split(" ");

                if (parts.length < 2) {
                    continue;
                }

                var set = {
                    key: parts[0],
                    type: parts.length > 2 ? parts[1] : null,
                    value: parts.length > 2 ? parts[2] : parts[1]
                };

                if (set.type) {
                    set.type = "string";
                }

                if (set.value) {
                    switch (set.type.toLowerCase()) {
                        case "i":
                        case "int":
                        case "integer":
                            // Parse value as integer
                            set.value = parseInt(set.value);
                            break;
                        case "f":
                        case "float":
                        case "single":
                        case "d":
                        case "double":
                        case "number":
                            // Parse value as float
                            set.value = parseFloat(set.value);
                            break;
                        case "b":
                        case "bool":
                        case "boolean":
                            set.value = set.value == "true";
                            break;
                    }

                    this.settings[set.key] = set.value;
                }

            }

        }

        f.close();

    } catch (err) {
        alert("Error reading config file: " + err);
    }

}
/**
 * Write wtf formatted config file
 */
ConfigFile.prototype.write = function () {

    try {

        var f = this.file;

        if (f.exists)
            f.remove();

        f.encoding = "UTF8";

        f.open('w');
        if (f.error && f.error.length > 0)
            return f.error;

        for (var key in this.settings) {
            var value = this.settings[key];
            f.writeln([key, typeof (value), value].join(' '));
        }

        f.write();
        if (f.error && f.error.length > 0)
            return f.error;

        f.close();
        if (f.error && f.error.length > 0)
            return f.error;

        return "";

    } catch (err) {
        alert("ConfigFile.prototype.write: " + err);
        return err;
    }

}

ConfigFile.prototype.getValue = function (key) {

    if (!(key in this.settings)) {
        return;
    }

    return this.settings[key];

}
ConfigFile.prototype.setValue = function (key, value) {

    this.settings[key] = value;

    if (this.autoWrite) {
        this.write();
    }

}

ConfigFile.open = function (path) {
    if (typeof path !== "string") {
        if (path instanceof File) {
            path = path.fullName;
        } else {
            path = BEAUTYPANEL.files.settings;
        }
    }
    return new ConfigFile(path);
};

ConfigFile.merge = function (a, b) {

  a = ConfigFile.open(a);
  b = ConfigFile.open(b);

  b.autoWrite = false;

  for (var key in a.settings) {
    if (!(key in b.settings)) {
      b.setValue(key, a.getValue(key));
    }
  }

  b.write();

};
